==> flyhard-0.3.tar.gz: flyhard-0.3/src/enemies.h <==
/*
    enemies.h - Unsavoury characters who make the player's life harder
    Copyright (C) 2006 Mark boyd
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be fun to play,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/
#include "movers.h"

class enemy:public mover
{
public:
  virtual int take_hit(int damage);
    
protected:
  enemy() {m_hitpoints=1;m_dam_red=0;m_score=0;}
  int m_hitpoints;
  int m_dam_red;
  int m_score;
    
private:
  virtual void notify_dead(){};

};

class turret:public enemy  //number 1
{
public:
  enum direction
  {
    UP,DOWN,LEFT,RIGHT
  };
  
  turret(direction);
  
  void do_stuff(float time_interval);
  SDL_Surface* pic() const {return m_pic;}
  void notify_dead();
  
private:
  SDL_Surface *m_pic;
  
  int m_direction;
  float m_reload;
};

class t_shot:public enemy
{
public:
  t_shot();
  
  void do_stuff(float time_interval);
  bool does_arena_collisions() const {return true;}
  void notify_collision(arena_t);
  SDL_Surface* pic() const {return m_pic;}
  
private:
  SDL_Surface *m_pic;
  float m_life_expectancy;
};

class bobbler:public enemy
{
public:
  bobbler();
  
  void do_stuff(float time_interval);
  SDL_Surface* pic() const {return m_pic;}
  void notify_dead();
private:
  SDL_Surface *m_pic;
  int m_phase;
};

class miner:public enemy
{
public:
  miner();  
  void detonate();
  
  void do_stuff(float time_interval);
  SDL_Surface* pic() const {return m_pic;}
  bool does_arena_collisions() const {return true;}
  bool does_collisions(mover *m) const;
  void notify_collision(arena_t);
  void notify_collision(mover *m);
  void notify_dead();

private:
  SDL_Surface *m_pic;
};


class graldor:public enemy
{
public: 
  graldor();
  
  void do_stuff(float time_interval);
  SDL_Surface* pic() const {return m_pic;}
  bool does_arena_collisions() const;
  void notify_collision(arena_t);
  void notify_dead();
  
private:
  SDL_Surface *m_pic;
  float m_reload;
  float m_firing;
};

class baby_graldor:public enemy
{
public: 
  baby_graldor();
  
  void do_stuff(float time_interval);
  SDL_Surface* pic() const {return m_pic;}
  bool does_arena_collisions() const {return true;}
  void notify_collision(arena_t);
  void notify_dead();

private:
  SDL_Surface *m_pic;
};


class invader:public enemy
{
public: 
  invader(float left, float right);
  void do_stuff(float time_interval);
  SDL_Surface* pic() const {return m_pic;}
  void notify_dead();


private:
  float m_timer;
  float m_left;
  float m_right;
  float m_dir;
  SDL_Surface *m_pic;

};

class ufo:public enemy
{
public:
  //UFO likes to be at a certain height, and moves horizontally between "left" and "right"
  ufo(float desired_y, int left, int right);
  SDL_Surface* pic() const {return m_pic;}
  void do_stuff(float time_interval);
  void notify_dead();

private:
  SDL_Surface *m_pic;
  float m_timer;

  float m_desired_y;
  float m_left;
  float m_right;
};

class ufo_rocket:public enemy
{
public:
  ufo_rocket();
  SDL_Surface* pic() const {return m_pic;}
  bool does_arena_collisions() const {return true;}
  void notify_collision(arena_t);
  void notify_dead();

private:
  SDL_Surface *m_pic;

};

==> flyhard-0.42.tar.gz: flyhard-0.42/src/enemies.h <==
/*
    enemies.h - Unsavoury characters who make the player's life harder
    Copyright (C) 2006,2008 Mark Boyd
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be fun to play,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/
#include "movers.h"

class enemy:public mover
{
public:
  //This returns the amount of damage absorbed.
  //e.g. if "damage" is  3, we have 2 hitpoints and DR 4, return 3 and no change
  //e.g. if "damage" is  5, we have 2 hitpoints and DR 4, return 5 and reduce HP to 1
  //e.g. if "damage" is 10, we have 2 hitpoints and DR 4, return 6 and die
  virtual double take_hit(double damage);
  double insta_kill();  //kills this enemy, as if it had taken sufficient damage via take_hit 

  int dam_red(){return m_dam_red;}

  //"boss" status grants immunity to gimmicks (like the scoop) which make enemies all too easy to kill.
  virtual bool is_boss() const {return false;}

protected:
  enemy() {m_hitpoints=1;m_dam_red=0;m_score=0;}
  double m_hitpoints;
  int m_dam_red;
  int m_score;
    
private:
  virtual void notify_dead(){};  //override this to do stuff (explode, drop coins, etc)

public:
  static void collide(enemy *e1, enemy *e2);
};

class turret:public enemy  //number 1
{
public:
  enum direction
  {
    UP,DOWN,LEFT,RIGHT
  };
  
  turret(direction);
  
  void do_stuff(float time_interval);
  SDL_Surface* pic() const {return m_pic;}
  void notify_dead();
  
private:
  SDL_Surface *m_pic;
  
  int m_direction;
  float m_reload;
};

class t_shot:public enemy
{
public:
  t_shot();
  
  void do_stuff(float time_interval);
  bool does_arena_collisions() const {return true;}
  void notify_collision(arena_t, float);
  SDL_Surface* pic() const {return m_pic;}
  
private:
  SDL_Surface *m_pic;
  float m_life_expectancy;
};

class bobbler:public enemy
{
public:
  bobbler();
  
  void do_stuff(float time_interval);
  SDL_Surface* pic() const {return m_pic;}
  void notify_dead();
private:
  SDL_Surface *m_pic;
  int m_phase;
};

class miner:public enemy
{
public:
  miner();  
  void detonate();
  
  void do_stuff(float time_interval);
  SDL_Surface* pic() const {return m_pic;}
  bool does_arena_collisions() const {return true;}
  bool does_collisions(mover *m) const;
  void notify_collision(arena_t, float);
  void notify_collision(mover *m, float);
  void notify_dead();

private:
  SDL_Surface *m_pic;
};


class graldor:public enemy
{
public: 
  graldor();
  
  void do_stuff(float time_interval);
  SDL_Surface* pic() const {return m_pic;}
  bool does_arena_collisions() const;
  void notify_collision(arena_t, float);
  void notify_dead();
  bool is_stuck() const {return m_stuck;}
  
private:
  SDL_Surface *m_pic;
  float m_reload;
  float m_firing;
  bool m_stuck;
};

class baby_graldor:public enemy
{
public: 
  baby_graldor();
  
  void do_stuff(float time_interval);
  SDL_Surface* pic() const {return m_pic;}
  bool does_arena_collisions() const {return true;}
  void notify_collision(arena_t, float);
  void notify_dead();

private:
  SDL_Surface *m_pic;
};


class invader:public enemy
{
public: 
  invader(float left, float right);
  void do_stuff(float time_interval);
  SDL_Surface* pic() const {return m_pic;}
  void notify_dead();


private:
  float m_timer;
  float m_left;
  float m_right;
  float m_dir;
  SDL_Surface *m_pic;

};

class ufo:public enemy
{
public:
  //UFO likes to be at a certain height, and moves horizontally between "left" and "right"
  ufo(float desired_y, int left, int right);
  SDL_Surface* pic() const {return m_pic;}
  void do_stuff(float time_interval);
  void notify_dead();
  bool is_boss() const {return true;}

private:
  SDL_Surface *m_pic;
  float m_timer;

  float m_desired_y;
  float m_left;
  float m_right;
};

class ufo_rocket:public enemy
{
public:
  ufo_rocket();
  SDL_Surface* pic() const {return m_pic;}
  bool does_arena_collisions() const {return true;}
  void notify_collision(arena_t, float);
  void notify_dead();

private:
  SDL_Surface *m_pic;

};


class asteroid:public enemy
{
public:
  asteroid();
  SDL_Surface* pic() const {return m_pic;}
  void do_stuff(float time_interval);
  bool does_arena_collisions() const {return true;}
  void notify_collision(arena_t, float);
  void notify_dead();
private:
  enum size{AS_LARGE, AS_MEDIUM, AS_SMALL};

  asteroid(size);
  size m_size;
  SDL_Surface *m_pic;
};

class ast_ufo:public enemy
{
public:
  ast_ufo();
  SDL_Surface* pic() const {return m_pic;}
  void do_stuff(float time_interval);
  bool does_arena_collisions() const {return false;}
  void notify_dead();

private:
  SDL_Surface *m_pic;
};
